""" Webcropper for uni-potsdam.de/nachrichten """
# TESTED 15th of November 2021
# !!RUNTIME 4h 19min

import time
import requests as rq
from bs4 import BeautifulSoup as bsx


# helper
def get_scoop(rq_url):
    """ request and souping """
    res = rq.get(rq_url)
    return bsx(res.text, 'html.parser')


def get_categories(branch_prefix, branch_sets):
    """ getting categories """
    # setup
    print("starting the cropper")
    ls_curls = []

    # squeeze available categories
    print("squeeze available categories:")
    scoop = get_scoop(branch_prefix + str(1) + branch_sets + str(1))
    c_urls = list(map(lambda x: x['href'], scoop.findAll('a')))  # getting the category URLs
    c_urls = list(filter(lambda x: x.startswith("/de/nachrichten/page?tx_news_pi"), c_urls))
    c_names = list(filter(lambda x: x.text.startswith(" "), scoop.findAll('a')))  # getting names
    for url, name in zip(c_urls, c_names):
        ls_curls.append((url[70:], name.text[1: len(name.text) - 1]))
    print(str(len(ls_curls)) + " categories found")
    return ls_curls


def run_crop(categories, branch_prefix, branch_sets):
    """ processing categories """
    wait_index = 0
    for dv_url in categories[0:]:
        print("gathering on topic: " + dv_url[1])
        ls_urls, branch_page = [], 1

        # getting relevant URLS
        while True:
            print("getting available links on page " + str(branch_page))
            branch_url = branch_prefix + str(branch_page) + branch_sets + dv_url[0]
            scoop = get_scoop(branch_url)
            t_urls = list(map(lambda x: x['href'], scoop.findAll('a')))
            t_urls = list(filter(lambda x: x.startswith("/de/nachrichten/detail"), t_urls))
            if not t_urls:
                break
            t_last = ""
            for t_url in t_urls:
                if t_last != t_url:
                    ls_urls.append('https://www.uni-potsdam.de' + t_url)
                t_last = t_url
            branch_page += 1

        # work with found articles
        print("\nstarting to read the found articles")
        wait_index = read_articles(ls_urls, dv_url, wait_index)


def read_articles(articles, category, wait_index):
    """ read and process articles """
    for url in articles:
        print("now reading: " + url)

        # respectfully waiting if necessary
        if wait_index > 200:
            print("respectfully waiting for 5 minutes...")
            time.sleep(300)
            wait_index = 0
        else:
            wait_index += 1

        # compiling article
        scoop, s_article = get_scoop(url), ""
        title = scoop.find('h1')
        s_article += title.text + "\\n\\n"
        paragraphs = scoop.findAll('p')
        for paragraph in paragraphs[1:]:
            s_article += paragraph.text + "\\n"

        # write to csv
        with open("export_raw.csv", "a", encoding='UTF-8') as tch:
            tch.write(category[1] + "," + s_article + "\n")
    return wait_index


def join_categories():
    """ join duplicated articles from different categories """
    print("\nsearching & joining duplicates...")
    with open("export_raw.csv", "r", encoding='UTF-8') as rfile:
        lines = list(map(lambda x: x.split(',', 1), rfile.readlines()))
        for j, line in enumerate(lines):
            i = j + 1
            while i < len(lines):
                if line[1] == lines[i][1]:
                    print("joining duplicate between \"" + line[0] + "\" and \"" + lines[i][0] + "\"")
                    line[0] += ';' + lines[i][0]
                    lines.pop(i)
                else:
                    i += 1

    # rewrite joined information as updated csv
    print("\nwriting the final .csv file...")
    with open("export.csv", "a", encoding='UTF-8') as rwr:
        for line in lines:
            rwr.write(line[0] + ',' + line[1])


def run():
    """ runs the entire webmediacropping process """
    prefix = 'https://www.uni-potsdam.de/de/nachrichten/page-'
    sets = '?tx_news_pi1[overwriteDemand][categories]='
    run_crop(get_categories(prefix, sets), prefix, sets)
    join_categories()
