""" Views """

import logging
import os.path
from django.shortcuts import render
from django.views import View
from prediction.train_data import train_model_single_labels
from prediction.tasks import train_model, pred_articles, assign_colors


logger = logging.getLogger(__name__)


class Trainer(View):
    """ Model Training Page """
    template_name = 'prediction/basic.html'

    def get(self, request, *args, **kwargs):
        """ Answer to GET request """
        logger.info("GET Simulator Prediction")
        return render(request, self.template_name)

    def post(self, request, *args, **kwargs):
        """ Answer to GET request """
        logger.info("POST Simulator Prediction")
        file_path = '/app/prediction/export.csv'

        if 'train_single' in request.POST:
            logger.info('Train Single')
            if os.path.isfile(file_path):
                train_model_single_labels(file_path)
                logger.info('Single Label Prediction: Done.')
            else:
                logger.info('File not found.')

        elif 'train_multiple' in request.POST:
            logger.info('Train Multiple')
            train_model.delay()
            logger.info('Multi Label Prediction: Done.')

        elif 'pred_multiple' in request.POST:
            logger.info('Predict Multitple')
            pred_articles.delay()
            logger.info('Multi Label Prediction: Done.')

        elif 'assign_colors' in request.POST:
            logger.info('Assign Colors')
            assign_colors.delay()

        return render(request, self.template_name)
